@@ -1,18 +1,17 @@
 # Copyright (C) 2003-2008 Brailcom, o.p.s.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation; either version 2.1 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 """Python API to Speech Dispatcher
 
@@ -149,7 +148,7 @@ class CommunicationMethod(object):
     """Inet socket communication using a host and port"""
 
 class _SSIP_Connection(object):
-    """Implemantation of low level SSIP communication."""
+    """Implementation of low level SSIP communication."""
     
     _NEWLINE = b"\r\n"
     _END_OF_DATA_MARKER = b'.'
@@ -198,7 +197,8 @@ class _SSIP_Connection(object):
         self._ssip_reply_semaphore = threading.Semaphore(0)
         self._communication_thread = \
                 threading.Thread(target=self._communication, kwargs={},
-                                 name="SSIP client communication thread")
+                                 name="SSIP client communication thread",
+                                 daemon=True)
         self._communication_thread.start()
     
     def close(self):
@@ -214,9 +214,9 @@ class _SSIP_Connection(object):
         self._communication_thread.join()
         
     def _communication(self):
-        """Handle incomming socket communication.
+        """Handle incoming socket communication.
 
-        Listens for all incomming communication on the socket, dispatches
+        Listens for all incoming communication on the socket, dispatches
         events and puts all other replies into self._com_buffer list in the
         already parsed form as (code, msg, data).  Each time a new item is
         appended to the _com_buffer list, the corresponding semaphore
@@ -290,7 +290,7 @@ class _SSIP_Connection(object):
         and return the triplet (code, msg, data)."""
         # TODO: This check is dumb but seems to work.  The main thread
         # hangs without it, when the Speech Dispatcher connection is lost.
-        if not self._communication_thread.isAlive():
+        if not self._communication_thread.is_alive():
             raise SSIPCommunicationError
         self._ssip_reply_semaphore.acquire()
         # The list is sorted, read the first item
@@ -456,7 +456,9 @@ class PunctuationMode(object):
     NONE = 'none'
     """Don't read any punctuation character at all."""
     SOME = 'some'
-    """Only the user-defined punctuation characters are read.
+    """Only some of the user-defined punctuation characters are read."""
+    MOST = 'most'
+    """Only most of the user-defined punctuation characters are read.
 
     The set of characters is specified in Speech Dispatcher configuration.
 
@@ -508,7 +510,7 @@ class SSIPClient(object):
           component -- connection identification string.  When one client opens
             multiple connections, this can be used to identify each of them.
           user -- user identification string (user name).  When multi-user
-            acces is expected, this can be used to identify their connections.
+            access is expected, this can be used to identify their connections.
           address -- server address as specified in Speech Dispatcher
             documentation (e.g. "unix:/run/user/joe/speech-dispatcher/speechd.sock"
             or "inet:192.168.0.85:6561")
@@ -665,13 +667,16 @@ class SSIPClient(object):
                     "resolves on %s which seems to be a remote host. You must start the "
                     "server manually or choose another connection address." % (connection_args['host'],
                                                                                str(ip_addresses),))
-        if os.path.exists(paths.SPD_SPAWN_CMD):
+        cmd = os.getenv("SPEECHD_CMD")
+        if not cmd:
+            cmd = paths.SPD_SPAWN_CMD
+        if os.path.exists(cmd):
             connection_params = []
             for param, value in connection_args.items():
                 if param not in ["host",]:
                     connection_params += ["--"+param.replace("_","-"), str(value)]
 
-            server = subprocess.Popen([paths.SPD_SPAWN_CMD, "--spawn"]+connection_params,
+            server = subprocess.Popen([cmd, "--spawn"]+connection_params,
                                       stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             stdout_reply, stderr_reply = server.communicate()
             retcode = server.wait()
@@ -680,7 +685,7 @@ class SSIPClient(object):
             return server.pid
         else:
             raise SpawnError("Can't find Speech Dispatcher spawn command %s"
-                                         % (paths.SPD_SPAWN_CMD,))
+                                         % (cmd))
 
     def set_priority(self, priority):
         """Set the priority category for the following messages.
@@ -840,10 +845,10 @@ class SSIPClient(object):
     def list_synthesis_voices(self):
         """Return names of all available voices for the current output module.
 
-        Returns a tuple of tripplets (name, language, dialect).
+        Returns a tuple of tripplets (name, language, variant).
 
-        'name' is a string, 'language' is an ISO 639-1 Alpha-2 language code
-        and 'dialect' is a string.  Language and dialect may be None.
+        'name' is a string, 'language' is an ISO 639-1 Alpha-2/3 language code
+        and 'variant' is a string.  Language and variant may be None.
 
         """
         try:
@@ -851,21 +856,28 @@ class SSIPClient(object):
         except SSIPCommandError:
             return ()
         def split(item):
-            name, lang, dialect = tuple(item.rsplit(' ', 3))
-            return (name, lang or None, dialect or None)
+            name, lang, variant = tuple(item.rsplit('\t', 3))
+            return (name, lang or None, variant or None)
         return tuple([split(item) for item in data])
 
     def set_language(self, language, scope=Scope.SELF):
         """Switch to a particular language for further speech commands.
 
         Arguments:
-          language -- two letter language code according to RFC 1776 as string.
+          language -- two/three letter language code according to RFC 1766 as string, possibly with a region qualification.
           scope -- see the documentation of this class.
             
         """
-        assert isinstance(language, str) and len(language) == 2
+        assert isinstance(language, str)
         self._conn.send_command('SET', scope, 'LANGUAGE', language)
 
+    def get_language(self):
+        """Get the current language."""
+        code, msg, data = self._conn.send_command('GET', 'LANGUAGE')
+        if data:
+            return data[0]
+        return None
+
     def set_output_module(self, name, scope=Scope.SELF):
         """Switch to a particular output module.
 
@@ -876,6 +888,13 @@ class SSIPClient(object):
         """
         self._conn.send_command('SET', scope, 'OUTPUT_MODULE', name)
 
+    def get_output_module(self):
+        """Get the current output module."""
+        code, msg, data = self._conn.send_command('GET', 'OUTPUT_MODULE')
+        if data:
+            return data[0]
+        return None
+
     def set_pitch(self, value, scope=Scope.SELF):
         """Set the pitch for further speech commands.
 
@@ -890,6 +909,27 @@ class SSIPClient(object):
         assert isinstance(value, int) and -100 <= value <= 100, value
         self._conn.send_command('SET', scope, 'PITCH', value)
 
+    def get_pitch(self):
+        """Get the current pitch."""
+        code, msg, data = self._conn.send_command('GET', 'PITCH')
+        if data:
+            return data[0]
+        return None
+
+    def set_pitch_range(self, value, scope=Scope.SELF):
+        """Set the pitch range for further speech commands.
+
+        Arguments:
+          value -- integer value within the range from -100 to 100, with 0
+            corresponding to the default pitch range of the current speech synthesis
+            output module, lower values meaning lower pitch range and higher values
+            meaning higher pitch range.
+          scope -- see the documentation of this class.
+          
+        """
+        assert isinstance(value, int) and -100 <= value <= 100, value
+        self._conn.send_command('SET', scope, 'PITCH_RANGE', value)
+
     def set_rate(self, value, scope=Scope.SELF):
         """Set the speech rate (speed) for further speech commands.
 
@@ -904,6 +944,13 @@ class SSIPClient(object):
         assert isinstance(value, int) and -100 <= value <= 100
         self._conn.send_command('SET', scope, 'RATE', value)
 
+    def get_rate(self):
+        """Get the current speech rate (speed)."""
+        code, msg, data = self._conn.send_command('GET', 'RATE')
+        if data:
+            return data[0]
+        return None
+
     def set_volume(self, value, scope=Scope.SELF):
         """Set the speech volume for further speech commands.
 
@@ -917,23 +964,37 @@ class SSIPClient(object):
         assert isinstance(value, int) and -100 <= value <= 100
         self._conn.send_command('SET', scope, 'VOLUME', value)
 
+    def get_volume(self):
+        """Get the speech volume."""
+        code, msg, data = self._conn.send_command('GET', 'VOLUME')
+        if data:
+            return data[0]
+        return None
+
     def set_punctuation(self, value, scope=Scope.SELF):
-        """Set the punctuation pronounciation level.
+        """Set the punctuation pronunciation level.
 
         Arguments:
           value -- one of the 'PunctuationMode' constants.
           scope -- see the documentation of this class.
             
         """
-        assert value in (PunctuationMode.ALL, PunctuationMode.SOME,
-                         PunctuationMode.NONE), value
+        assert value in (PunctuationMode.ALL, PunctuationMode.MOST,
+                         PunctuationMode.SOME, PunctuationMode.NONE), value
         self._conn.send_command('SET', scope, 'PUNCTUATION', value)
 
+    def get_punctuation(self):
+        """Get the punctuation pronunciation level."""
+        code, msg, data = self._conn.send_command('GET', 'PUNCTUATION')
+        if data:
+            return data[0]
+        return None
+
     def set_spelling(self, value, scope=Scope.SELF):
-        """Toogle the spelling mode or on off.
+        """Toggle the spelling mode or on off.
 
         Arguments:
-          value -- if 'True', all incomming messages will be spelled
+          value -- if 'True', all incoming messages will be spelled
             instead of being read as normal words. 'False' switches
             this behavior off.
           scope -- see the documentation of this class.
@@ -951,7 +1012,7 @@ class SSIPClient(object):
         Arguments:
           value -- one of 'none', 'spell', 'icon'. None means no signalization
             of capital letters, 'spell' means capital letters will be spelled
-            with a syntetic voice and 'icon' means that the capital-letter icon
+            with a synthetic voice and 'icon' means that the capital-letter icon
             will be prepended before each capital letter.
           scope -- see the documentation of this class.
             
@@ -976,7 +1037,7 @@ class SSIPClient(object):
                value.lower() in ("male1", "male2", "male3", "female1",
                                  "female2", "female3", "child_male",
                                  "child_female")
-        self._conn.send_command('SET', scope, 'VOICE', value)
+        self._conn.send_command('SET', scope, 'VOICE_TYPE', value)
 
     def set_synthesis_voice(self, value, scope=Scope.SELF):
         """Set voice by its real name.
@@ -1008,7 +1069,7 @@ class SSIPClient(object):
         written into these files with maximal verbosity until switched
         off. You should always first call set_debug_destination.
 
-        The intended use of this functionality is to switch debuging
+        The intended use of this functionality is to switch debugging
         on for a period of time while the user will repeat the behavior
         and then send the logs to the appropriate bug-reporting place.
 
@@ -1031,7 +1092,7 @@ class SSIPClient(object):
         """Set debug destination.
 
         Arguments:
-          path -- path (string) to the directory where debuging
+          path -- path (string) to the directory where debugging
                   files will be created
           scope -- see the documentation of this class.
         
@@ -1066,7 +1127,7 @@ class SSIPClient(object):
 class Client(SSIPClient):
     """A DEPRECATED backwards-compatible API.
 
-    This Class is provided only for backwards compatibility with the prevoius
+    This Class is provided only for backwards compatibility with the previous
     unofficial API.  It will be removed in future versions.  Please use either
     'SSIPClient' or 'Speaker' interface instead.  As deprecated, the API is no
     longer documented.
