@@ -43,7 +43,7 @@ static pthread_t generic_speak_thread;
 static pid_t generic_pid;
 static sem_t generic_semaphore;
 
-static char **generic_message;
+static char *generic_message;
 static SPDMessageType generic_message_type;
 
 static int generic_position = 0;
@@ -52,6 +52,8 @@ static int generic_pause_requested = 0;
 static char *execute_synth_str1;
 static char *execute_synth_str2;
 
+static gboolean initialized = FALSE;
+
 /* Internal functions prototypes */
 static void *get_ht_option(GHashTable * hash_table, const char *key);
 static void *_generic_speak(void *);
@@ -71,6 +73,8 @@ void generic_set_punct(SPDPunctuation punct);
 
 MOD_OPTION_1_STR(GenericExecuteSynth)
     MOD_OPTION_1_STR(GenericCmdDependency)
+    MOD_OPTION_1_INT(GenericPortDependency)
+    MOD_OPTION_1_STR(GenericSoundIconFolder)
 
     MOD_OPTION_1_INT(GenericMaxChunkLength)
     MOD_OPTION_1_STR(GenericDelimiters)
@@ -111,6 +115,8 @@ int module_load(void)
 
 	MOD_OPTION_1_STR_REG(GenericExecuteSynth, "");
 	MOD_OPTION_1_STR_REG(GenericCmdDependency, "");
+	MOD_OPTION_1_INT_REG(GenericPortDependency, 0);
+	MOD_OPTION_1_STR_REG(GenericSoundIconFolder, "/usr/share/sounds/sound-icons/");
 
 	REGISTER_DEBUG();
 
@@ -154,10 +160,18 @@ int module_init(char **status_info)
 
 	*status_info = NULL;
 
+	if (module_list_registered_voices() == NULL)
+	{
+		*status_info = g_strdup("The module does not have any voice configured, "
+					"please add them in the configuration file, "
+					"or install the required files");
+		return -1;
+	}
 	DBG("GenericMaxChunkLength = %d\n", GenericMaxChunkLength);
 	DBG("GenericDelimiters = %s\n", GenericDelimiters);
 	DBG("GenericExecuteSynth = %s\n", GenericExecuteSynth);
 	DBG("GenericCmdDependency = %s\n", GenericCmdDependency);
+	DBG("GenericPortDependency = %u\n", GenericPortDependency);
 
 	generic_msg_language =
 	    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
@@ -165,22 +179,23 @@ int module_init(char **status_info)
 	generic_msg_language->charset = g_strdup("iso-8859-1");
 	generic_msg_language->name = g_strdup("english");
 
-	generic_message = g_malloc(sizeof(char *));
+	generic_message = NULL;
 
 	sem_init(&generic_semaphore, 0, 0);
 
 	DBG("Generic: creating new thread for generic_speak\n");
 	generic_speaking = 0;
-	ret = pthread_create(&generic_speak_thread, NULL, _generic_speak, NULL);
+	ret = spd_pthread_create(&generic_speak_thread, NULL, _generic_speak, NULL);
 	if (ret != 0) {
 		DBG("Generic: thread failed\n");
-		*status_info = g_strdup("The module couldn't initialize threads"
-					"This can be either an internal problem or an"
-					"architecture problem. If you are sure your architecture"
+		*status_info = g_strdup("The module couldn't initialize threads "
+					"This can be either an internal problem or an "
+					"architecture problem. If you are sure your architecture "
 					"supports threads, please report a bug.");
 		return -1;
 	}
 
+	initialized = TRUE;
 	*status_info = g_strdup("Everything ok so far.");
 	return 0;
 }
@@ -200,9 +215,9 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 		DBG("Speaking when requested to write");
 		return 0;
 	}
-	UPDATE_STRING_PARAMETER(voice.name, generic_set_synthesis_voice);
 	UPDATE_STRING_PARAMETER(voice.language, generic_set_language);
 	UPDATE_PARAMETER(voice_type, generic_set_voice);
+	UPDATE_STRING_PARAMETER(voice.name, generic_set_synthesis_voice);
 	UPDATE_PARAMETER(punctuation_mode, generic_set_punct);
 	UPDATE_PARAMETER(pitch, generic_set_pitch);
 	UPDATE_PARAMETER(pitch_range, generic_set_pitch_range);
@@ -223,7 +238,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	} else {
 		DBG("Warning: Preferred charset not specified, recoding to iso-8859-1");
 		tmp =
-		    (char *)g_convert_with_fallback(data, bytes, "iso-8859-2",
+		    (char *)g_convert_with_fallback(data, bytes, "iso-8859-1",
 						    "UTF-8",
 						    GenericRecodeFallback, NULL,
 						    NULL, NULL);
@@ -234,16 +249,16 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 
 	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	if (msgtype == SPD_MSGTYPE_TEXT)
-		*generic_message = module_strip_ssml(tmp);
+		generic_message = module_strip_ssml(tmp);
 	else
-		*generic_message = g_strdup(tmp);
+		generic_message = g_strdup(tmp);
 	g_free(tmp);
 
-	module_strip_punctuation_some(*generic_message, GenericStripPunctChars);
+	module_strip_punctuation_some(generic_message, GenericStripPunctChars);
 
-	generic_message_type = SPD_MSGTYPE_TEXT;
+	generic_message_type = msgtype;
 
-	DBG("Requested data: |%s|\n", data);
+	DBG("Requested data (%d): |%s|\n", msgtype, data);
 
 	/* Send semaphore signal to the speaking thread */
 	generic_speaking = 1;
@@ -291,11 +306,16 @@ int module_close(void)
 		module_stop();
 	}
 
+	if (!initialized)
+		return 0;
+
 	if (module_terminate_thread(generic_speak_thread) != 0)
 		return -1;
 
 	sem_destroy(&generic_semaphore);
 
+	initialized = FALSE;
+
 	return 0;
 }
 
@@ -325,7 +345,7 @@ char *string_replace(char *string, const char *token, const char *data)
 
 	mstring = g_strdup(string);
 	while (1) {
-		/* Split the string in two parts, ommit the token */
+		/* Split the string in two parts, omit the token */
 		p = strstr(mstring, token);
 		if (p == NULL) {
 			return mstring;
@@ -351,12 +371,39 @@ void *_generic_speak(void *nothing)
 
 	DBG("generic: speaking thread starting.......\n");
 
+	/* Make interruptible */
 	set_speaking_thread_parameters();
 
 	while (1) {
 		sem_wait(&generic_semaphore);
 		DBG("Semaphore on\n");
 
+		const char *play_command = NULL;
+		play_command = spd_audio_get_playcmd(module_audio_id);
+
+		if (play_command == NULL) {
+			DBG("This audio backend has no default play command; using \"play\"\n");
+			play_command = "play";
+		}
+
+		if (generic_message_type == SPD_MSGTYPE_SOUND_ICON) {
+			if (strchr(generic_message, '\\') ||
+			    strchr(generic_message, '\'') ||
+			    strchr(generic_message, '/')) {
+				DBG("Warning: bad icon name %s\n", generic_message);
+			}
+			char *cmd = g_strdup_printf("%s '%s/%s'", play_command, GenericSoundIconFolder, generic_message);
+			module_report_event_begin();
+			DBG("icon command = |%s|\n", cmd);
+			ret = system(cmd);
+			if (ret)
+				DBG("failed to run icon command: (error=%d) %s\n", errno, strerror(errno));
+			module_report_event_end();
+			free(cmd);
+			generic_speaking = 0;
+			continue;
+		}
+
 		ret = pipe(module_pipe.pc);
 		if (ret != 0) {
 			DBG("Can't create pipe pc\n");
@@ -393,7 +440,6 @@ void *_generic_speak(void *nothing)
 				char *p;
 				char *tmpdir, *homedir;
 				const char *helper;
-				const char *play_command = NULL;
 
 				helper = getenv("TMPDIR");
 				if (helper)
@@ -408,13 +454,6 @@ void *_generic_speak(void *nothing)
 					homedir =
 					    g_strdup("UNKNOWN_HOME_DIRECTORY");
 
-				play_command =
-				    spd_audio_get_playcmd(module_audio_id);
-				if (play_command == NULL) {
-					DBG("This audio backend has no default play command; using \"play\"\n");
-					play_command = "play";
-				}
-
 				/* Set this process as a process group leader (so that SIGKILL
 				   is also delivered to the child processes created by system()) */
 				if (setpgid(0, 0) == -1)
@@ -454,10 +493,14 @@ void *_generic_speak(void *nothing)
 					e_string =
 					    string_replace(e_string, "$VOICE",
 							   generic_msg_voice_str);
-				else
+				else {
+					char *default_voice = module_getdefaultvoice();
+					if (!default_voice)
+						default_voice = "no_voice";
 					e_string =
 					    string_replace(e_string, "$VOICE",
-							   "no_voice");
+							    default_voice);
+				}
 
 				/* Cut it into two strings */
 				p = strstr(e_string, "$DATA");
@@ -481,7 +524,7 @@ void *_generic_speak(void *nothing)
 			/* This is the parent. Send data to the child. */
 
 			generic_position =
-			    module_parent_wfork(module_pipe, *generic_message,
+			    module_parent_wfork(module_pipe, generic_message,
 						generic_message_type,
 						GenericMaxChunkLength,
 						GenericDelimiters,
@@ -663,8 +706,7 @@ void generic_set_language(char *lang)
 		generic_msg_language->charset = NULL;
 		generic_msg_language->name = g_strdup(lang);
 	}
-
-	if (generic_msg_language->name == NULL) {
+	else if (generic_msg_language->name == NULL) {
 		DBG("Language name for %s not found in the configuration file.",
 		    lang);
 		generic_msg_language =
@@ -679,6 +721,7 @@ void generic_set_language(char *lang)
 
 void generic_set_voice(SPDVoiceType voice)
 {
+	DBG("Setting voice type %d", voice);
 	assert(generic_msg_language);
 	generic_msg_voice_str =
 	    module_getvoice(generic_msg_language->code, voice);
@@ -689,6 +732,7 @@ void generic_set_voice(SPDVoiceType voice)
 
 void generic_set_synthesis_voice(char *name)
 {
+	DBG("Setting voice name %s (%s)", name, msg_settings.voice.name);
 	assert(msg_settings.voice.name);
 	if (module_existsvoice(msg_settings.voice.name))
 		generic_msg_voice_str = msg_settings.voice.name;
