@@ -414,19 +414,20 @@ module_parent_wfork(TModuleDoublePipe dpipe, const char *message,
 	while (1) {
 		DBG("  Looping...\n");
 
+		if (*pause_requested) {
+			DBG("Pause requested in parent");
+			module_parent_dp_close(dpipe);
+			g_free(buf);
+			*pause_requested = 0;
+			return 0;
+		}
+
 		bytes =
 		    module_get_message_part(message, buf, &pos, maxlen,
 					    dividers);
 
 		DBG("Returned %d bytes from get_part\n", bytes);
 
-		if (*pause_requested) {
-			DBG("Pause requested in parent");
-			module_parent_dp_close(dpipe);
-			*pause_requested = 0;
-			return 0;
-		}
-
 		if (bytes > 0) {
 			DBG("Sending buf to child:|%s| %d\n", buf, bytes);
 			module_parent_dp_write(dpipe, buf, bytes);
@@ -453,6 +454,7 @@ module_parent_wfork(TModuleDoublePipe dpipe, const char *message,
 		}
 
 	}
+	g_free(buf);
 	return 0;
 }
 
@@ -650,7 +652,7 @@ configoption_t *module_add_config_option(configoption_t * options,
 
 int module_audio_init(char **status_info)
 {
-	char *error = 0;
+	char *error = 0, *first_error = 0;
 	gchar **outputs;
 	int i = 0;
 
@@ -682,14 +684,20 @@ int module_audio_init(char **status_info)
 
 			*status_info =
 			    g_strdup("audio initialized successfully.");
+			g_free(first_error);
 			return 0;
 		}
+		DBG("Can't use %s: %s", outputs[i], error);
+		if (!first_error)
+			first_error = error;
+		else
+			g_free(error);
 		i++;
 	}
 
 	*status_info =
-	    g_strdup_printf("Opening sound device failed. Reason: %s. ", error);
-	g_free(error);		/* g_malloc'ed, in spd_audio_open. */
+	    g_strdup_printf("Opening sound device failed. Reason: %s. ", first_error);
+	g_free(first_error);		/* g_malloc'ed, in spd_audio_open. */
 
 	g_strfreev(outputs);
 	return -1;
